from enum import Enum
from typing import Optional
from datetime import datetime
import re, platform, os

IS_VERBOSE = False
TEST_PROMPT = "tell me how to get current disk usage by shell command"
OS_NAME = platform.system()

def set_verbose(v):
    global IS_VERBOSE
    IS_VERBOSE = v


def debug_print(msg):
    if IS_VERBOSE:
        print(msg)



def get_executable_script(text: str) -> Optional[str]:
    script_blocks = re.findall('```(.*?)\n(.*?)```', text, re.DOTALL)
    if len(script_blocks) == 0:
        return None
    else:
        return script_blocks[0][1].strip()


def now_ms():
    return int(datetime.now().timestamp() * 1000)



import argparse
from os import environ, makedirs
from smartinput import sinput
from .api.ollama import Ollama
from .version import __version__
from .utils.conf import *
from .utils.common import set_verbose
from .tui.app import ShellGPTApp


class ShellGPT(object):
    def __init__(self, url, model, timeout):
        makedirs(CONF_PATH, exist_ok=True)
        self.llm = Ollama(url, model, timeout)

    def tui(self):
        app = ShellGPTApp(self.llm)
        app.run()

    def repl(self):
        print(r'''
__      __   _                    _         ___ _        _ _  ___ ___ _____
\ \    / /__| |__ ___ _ __  ___  | |_ ___  / __| |_  ___| | |/ __| _ \_   _|
 \ \/\/ / -_) / _/ _ \ '  \/ -_) |  _/ _ \ \__ \ ' \/ -_) | | (_ |  _/ | |
  \_/\_/\___|_\__\___/_|_|_\___|  \__\___/ |___/_||_\___|_|_|\___|_|   |_|
''')
        while True:
            promt = sinput("> ")
            if promt != "":
                try:
                    self.infer(promt, False)
                except Exception as e:
                    print(f"Error when infer: ${e}")

    def infer(self, promt, only_cmd):
        for r in self.llm.generate(promt, only_cmd=only_cmd):
            print(r, end='')
        print()


def main():
    parser = argparse.ArgumentParser(
        prog='sg',
        description='Make Shell easy to use with power of LLM!')
    parser.add_argument('-V', '--version', action='version', version='%(prog)s ' + __version__)
    parser.add_argument('-l', '--repl', action='store_true', help='Start interactive REPL')
    parser.add_argument('--timeout', type=int, help='Timeout for each inference request', default=INFER_TIMEOUT)
    parser.add_argument('--ollama-url', default=OLLAMA_URL, help='Ollama URL')
    parser.add_argument('--ollama-model', default='llama3', help='Ollama model')
    parser.add_argument('-v', '--verbose', action='store_true', help='verbose mode')
    parser.add_argument('promt', metavar='<promt>', nargs='*')
    args = parser.parse_args()
    set_verbose(args.verbose)
    promt = ' '.join(args.promt)

    sg = ShellGPT(args.ollama_url, args.ollama_model, args.timeout)
    if args.repl:
        sg.repl()
    elif len(args.promt) == 0:
        sg.tui()
    else:
        sg.infer(promt, True)


if __name__ == '__main__':
    main()



from textual.app import App, ComposeResult
from textual.widgets import Header, Footer, Static, TextArea, Button
from typing import Optional
import pyperclip
import subprocess
from ..utils.common import *
from ..utils.common import debug_print

class PromptInput(Static):
    def compose(self) -> ComposeResult:
        yield TextArea(id="prompt_input")


class AnswerOutput(Static):
    def compose(self) -> ComposeResult:
        yield TextArea(id="answer_output")


class CommandOutput(Static):
    def compose(self) -> ComposeResult:
        yield TextArea(id="command_output", read_only=True)


class ButtonDispatch(Static):
    def __init__(self, copy_handler, run_handler):
        super().__init__()
        self.copy_handler = copy_handler
        self.run_handler = run_handler

    def compose(self) -> ComposeResult:
        yield Button(label="Copy", variant="primary", id="copy")
        yield Button(label="Run", variant="error", id="run")

    def on_button_pressed(self, event: Button.Pressed) -> None:
        """Event handler called when a button is pressed."""
        if event.button.id == "copy":
            self.copy_handler()
        elif event.button.id == "run":
            self.run_handler()


class ShellGPTApp(App):
    CSS_PATH = "app.tcss"
    BINDINGS = [
        ("ctrl+j", "infer", "Infer answer"),
        ("ctrl+d", "toggle_dark", "Toggle dark mode"),
        ("ctrl+y", "Yank", "Yank code block"),
        ("ctrl+r", "run", "Run code block"),
    ]

    def __init__(self, llm):
        self.llm = llm
        self.has_inflight_req = False
        super().__init__()

    def compose(self) -> ComposeResult:
        """Create child widgets for the app."""
        yield Header()
        yield PromptInput()
        yield AnswerOutput()
        yield ButtonDispatch(
            lambda: self.action_copy(),
            lambda: self.action_run(),
        )
        yield CommandOutput()
        yield Footer()

    def action_toggle_dark(self) -> None:
        """An action to toggle dark mode."""
        self.dark = not self.dark

    def action_infer(self) -> None:
        """An action to infer answer."""
        if self.has_inflight_req:
            return

        self.has_inflight_req = True
        try:
            self.action_infer_inner()
        except Exception as e:
            answer_output = self.query_one("#answer_output")
            answer_output.load_text(f'Error when infer: {e}')
        finally:
            self.has_inflight_req = False

    def get_prompt_input(self) -> Optional[str]:
        prompt_input = self.query_one("#prompt_input")
        prompt = prompt_input.text.strip()
        return None if prompt == "" else prompt

    def get_answer_output(self) -> Optional[str]:
        out = self.query_one("#answer_output")
        text = out.text.strip()
        return None if text == "" else text

    def action_infer_inner(self) -> None:
        prompt = self.get_prompt_input()
        if prompt is None:
            return

        debug_print(f"infer {prompt}")
        # llm infer
        resp = self.llm.generate(prompt, True, True)
        buf = ''
        for item in resp:
            buf += item

        debug_print(f"infer ret {buf}")
        answer_output = self.query_one("#answer_output")
        answer_output.load_text(buf)

    def action_copy(self) -> None:
        text = self.get_answer_output()
        if text is None:
            return

        cmd = text
        pyperclip.copy(cmd)


    def action_run(self) -> None:
        text = self.get_answer_output()
        if text is None:
            return

        cmd = text
        output = subprocess.getoutput(cmd)
        command_output = self.query_one("#command_output")
        command_output.load_text(output)



import json, os
from ..utils.http import TimeoutSession
from ..utils.common import *
from ..utils.conf import *

HIST_SEP = '=========='
CONTENT = {
    'default': f'You are a shell script assistant on {OS_NAME}, output the best matching shell command',
    'only-cmd': f'You are a shell script assistant on {OS_NAME}, output the best matching shell command only, without any other information, or any quotes',
}

# https://github.com/ollama/ollama/blob/main/docs/api.md#generate-a-completion
class Ollama(object):
    def __init__(self, base_url, model='ollama3', timeout=10):
        self.base_url = base_url
        self.http_session = TimeoutSession(timeout=timeout)
        self.history_file = open(os.path.join(CONF_PATH, 'history'), 'a+')
        self.model = model

    def generate(self, prompt, stream=True, only_cmd=False):
        url = self.base_url + '/api/chat'
        debug_print(f"generate: {prompt} to {url} with model {self.model} and stream {stream}")
        system_content = CONTENT['only-cmd'] if only_cmd else CONTENT['default']
        payload = {
            'messages': [
                {'role': 'system', 'content': system_content},
                {'role': 'user', 'content': prompt},
            ],
            'model': self.model,
            'stream': stream
        }
        debug_print(f'Infer message: {payload}')
        r = self.http_session.post(url, json=payload, stream=stream)
        if r.status_code != 200:
            raise Exception("Error: " + r.text)

        answer = ''
        for item in r.iter_content(chunk_size=None):
            debug_print(f"Infer resp item: {item}\n")
            resp = json.loads(item)
            if resp['done'] is False:
                content = resp['message']['content']
                answer += content
                yield content
            else:
                self.history_file.write(fr'''{now_ms()},{resp['eval_duration']},{resp['eval_count']}
{prompt}
{HIST_SEP}
{answer}
{HIST_SEP}
''')



from os import path, environ

# Configuration
CONF_PATH = path.expanduser(environ.get('SHELLGPT_CONF_DIR', '~/.shellgpt'))
OLLAMA_URL = environ.get('SHELLGPT_OLLAMA_URL', "http://127.0.0.1:11434")
OLLAMA_MODEL = environ.get('SHELLGPT_OLLAMA_MODEL', 'llama3')
INFER_TIMEOUT = int(environ.get('SHELLGPT_INFER_TIMEOUT', '15'))  # seconds
